<?php
###########################################################
# Version history:                                        #
# - 2.0: Complete rewrite based on API version 4.2.3      #
###########################################################

###########################################################
# Supported features:                                     #
# - Setting login details from within WHMCS               #
# - Request EPP code                                      #
# - Request name servers                                  #
# - Change name servers                                   #
# - Setting registrar lock                                #
# - Transfer a domain                                     #
# - Register a domain                                     #
# - Renew a domain                                        #
#                                                         #
# Known missing features (as supported by WHMCS):         #
# - Getting registrar lock status                         #
# - Getting contacts for a domain                         #
###########################################################

###########################################################
# 01-10-2015											  #
# - Added sandbox mode 									  #
# - Added more TLD's to Transfer 						  #
# - Fixed account contacts bug       					  #
# - Added dns managment									  #
###########################################################

###########################################################
# 25-05-2016											  #
# - Bugfixes											  #
###########################################################

require_once("class_versio_api.php");
$versio_api = new Versio_API();

function versio_getConfigArray() {
	$configarray = array(
	 "Username" => array("Type" => "text", "Size" => "20", "Description" => "Enter your client number here", ),
	 "Password" => array("Type" => "password", "Size" => "20", "Description" => "Enter your password here", ),
	 "PasswordEncrypted" => array( "Type" => "yesno", "Description" => "Password encrypted with sha1", ),
	 "ContactFallback" => array( "Type" => "text", "Size" => "20", "Description" => "Standard contact ID if contact creation failed", ),
	 "EnableFallback" => array( "Type" => "yesno", "Description" => "Enable fallback contact", ),
	 "DisableRenewOnPayment" => array( "Type" => "yesno", "Description" => "Disable renew done by WHMCS once a payment is recieved, useful if autorenew is used", ),
	 "SandboxMode" => array("Type" => "yesno", "Description" => "Enable the sandbox mode don't allow any changes, use for testing", ),
	);
	return $configarray;
}

function versio_GetEPPCode($params) {
	
	# load the module
	$versio_api = new Versio_API();

	# setup command name
	$command_name = ($params['tld'] == 'nl' ? 'DomainsGetToken' : 'DomainsGetEPP');

	# setup command array
	$command = array(
		'command' => $command_name,
		'domain' => $params['sld'],
		'tld' => $params['tld']
	);

	# execute
	$versio = $versio_api->api_send($command,$params);

	# catch failed
	if($versio['success'] == 0)
		$values['error'] = 'Getting auth code not possible. '.$versio['command_response_code'].' '.$versio['command_response_message'];
	
	# set epp code if we have retrieved it from the versio
	$values['eppcode'] = ($versio['token'] ? $versio['token'] : '');

	# return the values
	return $values;
}



function versio_GetNameservers($params){
$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsListSingle",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	$versio = $versio_api->api_send($command,$params);
	if($versio['success'] !== "1"){
		$values['error'] = 'Getting name server information not possible. '.$versio['command_response_code'].' '.$versio['command_response_message'];
		$values["ns1"] = '';
		$values["ns2"] = '';
	    $values["ns3"] = '';
	    $values["ns4"] = '';
		$values["ns5"] = '';
	}else{
		$i = 1;
		if(isset($versio['ns1']) && !empty($versio['ns1'])){
			$values["ns".$i] = $versio['ns1'];
			$i++;
		}
		if(isset($versio['ns2']) && !empty($versio['ns2'])){
			$values["ns".$i] = $versio['ns2'];
			$i++;
		}
		
		
		if(isset($versio['ns3']) && !empty($versio['ns3'])){
			$values["ns".$i] = $versio['ns3'];
			$i++;
		}
		if(isset($versio['ns4']) && !empty($versio['ns4'])){
			$values["ns".$i] = $versio['ns4'];
			$i++;
		}
		if(isset($versio['ns5']) && !empty($versio['ns5'])){
			$values["ns".$i] = $versio['ns5'];
			$i++;
		} 
		if(isset($versio['ns6']) && !empty($versio['ns6'])){
			$values["ns".$i] = $versio['ns6'];
			$i++;
		}
		if(isset($versio['ns7']) && !empty($versio['ns7'])){
			$values["ns".$i] = $versio['ns7'];
			$i++;
		}
		if(isset($versio['ns8']) && !empty($versio['ns8'])){
			$values["ns".$i] = $versio['ns8'];
			$i++;
		}
		if(isset($versio['ns9']) && !empty($versio['ns9'])){
			$values["ns".$i] = $versio['ns9'];
		}
		$values['error'] = '';
	}
	return $values;
}

function versio_SaveNameservers($params) {
$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsSetNS",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	if(isset($params['ns1']) && !empty($params['ns1']) && $params['ns1'] !== ""){
		$command['ns1'] = $params['ns1'];
	}
	if(isset($params['ns2']) && !empty($params['ns2']) && $params['ns2'] !== ""){
		$command['ns2'] = $params['ns2'];
	}
	if(isset($params['ns3']) && !empty($params['ns3']) && $params['ns3'] !== ""){
		$command['ns3'] = $params['ns3'];
	}
	if(isset($params['ns4']) && !empty($params['ns4']) && $params['ns4'] !== ""){
		$command['ns4'] = $params['ns4'];
	}
	if(isset($params['ns5']) && !empty($params['ns5']) && $params['ns5'] !== ""){
		$command['ns5'] = $params['ns5'];
	}
	$versio = $versio_api->api_send($command,$params);
	if($versio['success'] !== "1"){
		return FALSE;
	}else{
		return TRUE;
	}
}
 
function versio_SaveRegistrarLock($params) {
	
	# load the api
	$versio_api = new Versio_API();

	# setup the command
	$command = array(
		'command' => 'DomainsSetLock',
		'domain' => $params['sld'],
		'tld' => $params['tld']
	);

	# locked or unlocked?
	$command['lock'] = ($params['lockenabled'] != 'unlocked' ? 1 : 0);

	# execute and handle error if one occurs
	$versio = $versio_api->api_send($command, $params);
	if($versio['success'] == 0)
		$values['error'] = 'Error: '. $versio['command_response_code'].' '.$versio['command_response_message'];

	# return result
	return $values;
}



function versio_SaveContactDetails($params) {

		# get the contact details
		$contactdetails = getContactDetails($params);


		# get the address
		$street = preg_replace('/^[0-9]+$/', '', $contactdetails['address1']);
		$hnr = preg_replace('/\D/', '', $contactdetails['address1']);

		# prepare the command
		unset($command);
		$command = array(
			'command' => 'DomainsCreateContact',
			'initials' => $contactdetails['firstname'],
			'surname' => $contactdetails['lastname'],
			'email' => $contactdetails['email'],
			'phone' => $contactdetails['phone'],
			'street' => $street,
			'hnr' => $hnr,
			'zipcode' => $contactdetails['zipcode'],
			'city' => $contactdetails['city'],
			'country' => $contactdetails['countrycode'],
			'companyname' => $contactdetails['companyname']
		);

		# init and make the api call
		$versio_api = new Versio_API();
		$versio = $versio_api->api_send($command,$params);

		# handle failed
		if($versio['success'] == 0) {
			$values['error'] = 'Creating contact failed. Versio API returned: '.$versio['command_response_code'].' '.$versio['command_response_message'];
			$values['status'] = false;
		}else{
			$values['error'] = '';
			$values['status'] = true;
			$values['id'] = $versio['contact_id'];
		}
	
		# return the values
		return $values;
}
/**
 * Get the contact details
 */
function getContactDetails($params)
{
	return array(
		'firstname' => $params['firstname'],
		'lastname' => $params['lastname'],
		'email' => $params['email'],
		'countryname' => $params['countryname'],
		'countrycode' => $params['countrycode'],
		'address1' => $params['address1'],
		'phone' => $params['phonenumber'],
		'zipcode' => $params['postcode'],
		'companyname' => $params['companyname'],
		'fullstate' => $params['fullstate'],
		'city' => $params['city']
	);
}


function versio_RegisterDomain($params) {

	# the account details
	$contactdetails = getContactDetails($params);

	# register the user
	$registrant = versio_SaveContactDetails($params);
	
	# check if registration is succesfull
	if($registrant['status'] === FALSE){
		if(isset($params['EnableFallback']) && !empty($params['EnableFallback']) && $params['EnableFallback'] == 'on')
			$registrant['id'] = $params['ContactFallback'];
		else
			$values['error'] = $registrant['error'];	
	}

	# errors? return them now
	if($values['error'])
		return $values;

	# load the api
	$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsRegister",
		"domain" => $params['sld'],
		"tld" => $params['tld'],
		"years" => $params['regperiod']
	);

	# set the nameservers
	logModuleCall("versio","versio.SaveContactDetails",$registrant,$values["error"],"",""); // log for debugging
	$park = 1;
	if(isset($params['ns1']) && !empty($params['ns1']) && $params['ns1'] !== ""){
		$command['ns1'] = $params['ns1'];
		$park = 0;
	} 
	if(isset($params['ns2']) && !empty($params['ns2']) && $params['ns2'] !== ""){
		$command['ns2'] = $params['ns2'];
		$park = 0;
	}
	if(isset($params['ns3']) && !empty($params['ns3']) && $params['ns3'] !== ""){
		$command['ns3'] = $params['ns3'];
		$park = 0;
	}
	if(isset($params['ns4']) && !empty($params['ns4']) && $params['ns4'] !== ""){
		$command['ns4'] = $params['ns4'];
		$park = 0;
	}
	if(isset($params['ns5']) && !empty($params['ns5']) && $params['ns5'] !== ""){
		$command['ns5'] = $params['ns5'];
		$park = 0;
	}

	$command['park'] = $park;
	$command['contact_id'] = $registrant['id'];
	
	# execute the script
	$versio = $versio_api->api_send($command,$params);
	
	# handles error	
	if($versio['success'] !== "1")
		$values['error'] = 'Versio: foutcode ' . $versio['command_response_code'].' - ' . $versio['command_response_message'];
	else
		$values['error'] = '';
		
	logModuleCall("versio","versio.RegisterDomain",$versio,$values["error"],"",""); // log for debugging
	return $values;
}

function versio_TransferDomain($params) {
	$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsTransfer",
		"domain" => $params['sld'],
		"tld" => $params['tld'],
		"years" => '1'
	);
	$park = 1;
	if(isset($params['ns1']) && !empty($params['ns1']) && $params['ns1'] !== ""){
		$command['ns1'] = $params['ns1'];
		$park = 0;
	}
	if(isset($params['ns2']) && !empty($params['ns2']) && $params['ns2'] !== ""){
		$command['ns2'] = $params['ns2'];
		$park = 0;
	}
	if(isset($params['ns3']) && !empty($params['ns3']) && $params['ns3'] !== ""){
		$command['ns3'] = $params['ns3'];
		$park = 0;
	}
	if(isset($params['ns4']) && !empty($params['ns4']) && $params['ns4'] !== ""){
		$command['ns4'] = $params['ns4'];
		$park = 0;
	}
	if(isset($params['ns5']) && !empty($params['ns5']) && $params['ns5'] !== ""){
		$command['ns5'] = $params['ns5'];
		$park = 0;
	}
	$command['park'] = $park;
	if(isset($params['transfersecret']) && !empty($params['transfersecret']) && $params['transfersecret'] !== ""){
		$command['code'] = $params['transfersecret'];
	}


	# save the contact
	$registrant = versio_SaveContactDetails($params);

	$error = 0;
	if($registrant['status'] === FALSE){
		if(isset($params['EnableFallback']) && !empy($params['EnableFallback']) && $params['EnableFallback'] === TRUE){
			$registrant['id'] = $params['ContactFallback'];
		}else{
			$values['error'] = 'Versio: foutcode ' . $registrant['command_response_code'].' - ' . $registrant['command_response_message'];
			$error = 1;
		}
	}
	$command['park'] = $park;
	if($error !== 1){
		$command['contact_id'] = $registrant['id'];
		$versio = $versio_api->api_send($command,$params);
		if($versio['success'] !== "1"){
			$values['error'] = 'Versio: foutcode ' . $versio['command_response_code'].' - ' . $versio['command_response_message'];
		}else{
			$values['error'] = ''; 
		}
	}
	return $values;
}

function versio_RenewDomain($params) {
	
	# prepare the command
	$command = array(
		"command" => "DomainsRenew",
		"domain" => $params['sld'],
		"tld" => $params['tld'],
		"years" => $params["regperiod"]
	);

	# load the api
	$versio_api = new Versio_API();

	if(isset($params['DisableRenewOnPayment']) && !empty($params['DisableRenewOnPayment']) && $params['DisableRenewOnPayment'] === TRUE){
		$values['error'] = '';
	}else{
		$versio = $versio_api->api_send($command,$params);
		if($versio['success'] !== "1"){
			$values['error'] = 'Versio: foutcode ' . $versio['command_response_code'].' - ' . $versio['command_response_message'];
		}else{
			$values['error'] = '';
		}
	}
	return $values;
}


function versio_GetContactDetailsType($id, $params) {
	$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsListSingleContact",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	$versio = $versio_api->api_send($command,$params);
	if($versio['success'] !== "1"){
		$values = array();
	}else{
		$values['Email'] = $versio['email'];
		$values['Fax'] = '';
		$values['ZIP Code'] = $versio['zipcode'];
		$values['Phone'] = $versio['phone'];
		$values['Country'] = $versio['country'];
		$values['State'] = '';
		$values['City'] = $versio['city'];
		$values['Street'] = $versio['street'].' '.$versio['hnr'].' '.$versio['hnradd'];
		$values['Organization'] = $versio['companyname'];
		$values['Last Name'] = $versio['surname'];
		$values['Middle Name'] = '';
		$value['First Name'] = $versio['initials'];
	}
	return $values;
}

function versio_GetDNS($params){
	$versio_api = new Versio_API();
	$command = array(
		"command" => "DomainsDNSListRecords",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	$versio = $versio_api->api_send($command,$params);
	

	if($versio['success'] !== "1"){
		$values = array();
	} else {
		if($versio['total_count']>0){
			$i = 1;
			while($versio['total_count']>=$i) {
				$values[$i]['recid']= $versio['id_'.$i];
				
				$values[$i]['hostname']= $versio['name_'.$i];
				$values[$i]['type']= $versio['type_'.$i];
				$values[$i]['address']= $versio['value_'.$i];
				$values[$i]['priority']= $versio['prio_'.$i];
				$values[$i]['ttl']= $versio['ttl_'.$i];
				
				
				$i++;
			}			
		}
		else {
			$values['error'] = 'No DNS Records found at Versio from this domain.';
		}
	}
	return $values;
}

function versio_SaveDNS($params){
	
	// dns records
	$records = $params['dnsrecords'];
	for($i=0;$i<sizeof($records); $i++){
		
		unset($versio, $command, $versio_api);
	
				if($records[$i]['address'] == '' && $records[$i]['recid'] !=  ''){

				$command =
				 array(
					"command" => "DomainsDNSDeleteRecord",
					"domain" => $params['sld'],
					"tld" => $params['tld'],
					'id' => $records[$i]['recid']
				);
				
	
				$versio_api = new Versio_API();
				$versio = $versio_api->api_send($command,$params);
			
			}elseif($records[$i]['hostname'] != '' && $records[$i]['type'] != '' && $records[$i]['address'] && $records[$i]['recid'] ==  ''){
		
				
				unset($command);
				if($records[$i]['priority'] == 'N/A')
					$records[$i]['priority'] = '';

				 $command = array(
					 "command" => "DomainsDNSAddRecord",
					 "domain" => $params['sld'],
					 "tld" => $params['tld'],
					 "name" => $records[$i]['hostname'],
					 "type" => $records[$i]['type'],
					 "value" => $records[$i]['address'],
					 "prio" => $records[$i]['priority'],
					 'ttl' => '14400'
				 );
	
				$versio_api = new Versio_API();
				$versio = $versio_api->api_send($command,$params);
		
		}else{
			
					unset($versio, $command, $versio_api);
						$command =
				 array(
					"command" => "DomainsDNSDeleteRecord",
					"domain" => $params['sld'],
					"tld" => $params['tld'],
					'id' => $records[$i]['recid']
				);
				
	
				$versio_api = new Versio_API();
				$versio = $versio_api->api_send($command,$params);
				
				if($records[$i]['priority'] == 'N/A')
					$records[$i]['priority'] = '';

				 $command = array(
					 "command" => "DomainsDNSAddRecord",
					 "domain" => $params['sld'],
					 "tld" => $params['tld'],
					 "name" => $records[$i]['hostname'],
					 "type" => $records[$i]['type'],
					 "value" => $records[$i]['address'],
					 "prio" => $records[$i]['priority'],
					 'ttl' => '14400'
				 );
				 
				$versio_api = new Versio_API();
				$versio = $versio_api->api_send($command,$params);
		}

	}
	$values['status'] = TRUE;
	$values['success'] = TRUE;
	$values['error'] = '';
	
	return $values;
}

function versio_GetContactDetails($params) {
	$values['error'] = 'Getting contact details not supported for this domain.';
	return $values;
}

function versio_GetRegistrarLock($params) {
	$values['error'] = 'Getting registrar lock not supported for this domain.';
	return $values;
}

function versio_Sync($params) {

//krijg domeinrenew waarde uit whmcs en update bij versio
$result = select_query("tbldomains", "donotrenew", array("registrar" => "versio", "id" => $params['domainid']));
if(mysql_num_rows($result) > 0) {
	while ($data = mysql_fetch_array($result)) {
		$donotrenew = $data['donotrenew'];
	}
}

if($donotrenew == '0')
{
$renew = '1';
}else{
$renew = '0';
}

unset($command);
	$command = array(
	"command" => "DomainsSetAutoRenew",
	"domain" => $params['sld'],
	"tld" => $params['tld'],
	"auto_renew" => $renew
	);
	
$values = array();

	$versio_api = new Versio_API();
	$versio = $versio_api->api_send($command,$params);

	IF($versio['success']==0) {
		$values['error'] = 'Fout opgetreden. Bekijk module log';
		logModuleCall("versio","Versio.updaterenew",$params['sld'].'.'.$params['tld'],$versio['command_response_message'],"",""); // log for debugging
	}
	else
	{
		logModuleCall("versio","Versio.updaterenew",$params['sld'].'.'.$params['tld'],"Auto renew geupdate bij versio","",""); // log for debugging
	}
	
//end domeinrenew
//krijg domein status
unset($command);
	$command = array(
	"command" => "DomainsListSingle",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	$versio = $versio_api->api_send($command,$params);

	IF($versio['success']==0) {
		$values['error'] = 'Fout opgetreden. Bekijk module log';
		logModuleCall("versio","Versio.Sync",$params['sld'].'.'.$params['tld'],$versio['command_response_message'],"",""); // log for debugging
	}	
	else
	{
		IF($versio['status']=='Actief') {
			
			$values['active'] = true;
			$date = date('Y-m-d',$versio['expiration_stamp']);
			$values['expirydate'] = $date;
		}
		else
		{
			$values['expired'] = true;
			$date = date('Y-m-d',$versio['expiration_stamp']);
			$values['expirydate'] = $date;
		}
	}

return $values; # return the details of the sync
}

function versio_TransferSync($params) {
//krijg domein status
unset($command);
	$command = array(
	"command" => "DomainsListSingle",
		"domain" => $params['sld'],
		"tld" => $params['tld']
	);
	$versio = $versio_api->api_send($command,$params);

	IF($versio['success']==0) {
		$values['error'] = 'Fout opgetreden. Bekijk module log';
		logModuleCall("versio","Versio.Sync",$params['sld'].'.'.$params['tld'],$versio['command_response_message'],"",""); // log for debugging
	}
	else
	{
		IF($versio['status']=='Actief') {
			
			$values['completed'] = true;
			$date = date('Y-m-d',$versio['expiration_stamp']);
			$values['expirydate'] = $date;
		}
	}

return $values; # return the details of the sync
}